@@ -206,6 +206,26 @@ def parse_gcov_file(fobj):
             coverage.append(int(cov_num))
     return coverage
 
+def combine_reports(original, new):
+    """Combines two gcov reports for a file into one by adding the number of hits on each line
+    """
+    if original == None:
+        return new
+    report = {}
+    report['name'] = original['name']
+    report['source'] = original['source']
+    coverage = []
+    for original_num, new_num in zip(original['coverage'], new['coverage']):
+        if original_num == None:
+            coverage.append(new_num)
+        elif new_num == None:
+            coverage.append(original_num)
+        else:
+            coverage.append(original_num + new_num)
+        
+    report['coverage'] = coverage
+    return report
+
 
 def collect_non_report_files(args, discovered_files):
     """Collects the source files that have no coverage reports.
@@ -249,7 +269,7 @@ def collect(args):
     report['service_job_id'] = args.service_job_id
 
     discovered_files = set()
-    report['source_files'] = []
+    src_files = {}
     abs_root = os.path.abspath(args.root)
     for root, dirs, files in os.walk(args.root):
         dirs[:] = filter_dirs(root, dirs, excl_paths)
@@ -291,8 +311,12 @@ def collect(args):
                         src_report['source'] = src_file.read()
 
                     src_report['coverage'] = parse_gcov_file(fobj)
-                    report['source_files'].append(src_report)
+                    if src_path in src_files:
+                        src_files[src_path] = combine_reports(src_files[src_path], src_report)
+                    else:
+                        src_files[src_path] = src_report
 
+    report['source_files'] = list(src_files.values())
     # Also collects the source files that have no coverage reports.
     report['source_files'].extend(
         collect_non_report_files(args, discovered_files))
